@@ -200,7 +200,7 @@ class Dataset(torch.utils.data.Dataset):
         if not isinstance(self._x, torch.Tensor):
             raise TypeError('Only datasets containing \'torch.Tensor\' input data can be normalized.')
 
-        self._x = scale(self._x)
+        self._x = torch.tensor(scale(self._x), dtype=torch.float)
 
     def complete(self) -> Dataset:
         """
