@@ -5,6 +5,7 @@ import torch
 from typing import Union
 from copy import deepcopy
 from ailca.core.env import *
+from ailca.core.operation import flatten_lists
 from ailca.data.base import Dataset
 from ailca.data.util import get_data_loader
 from ailca.data.multimodal import MultimodalDataset
@@ -162,8 +163,11 @@ def k_fold_cross_validation(k_folds: list,
                             loss_func: str = 'mae',
                             n_epochs: int = 300,
                             random_seed: int = 0):
-    preds = list()
     scores = list()
+    preds = list()
+    targets = list()
+    tooltips = list()
+    eval_results = dict()
 
     for k in range(0, len(k_folds)):
         dataset_train, dataset_test = k_folds[k]
@@ -172,8 +176,6 @@ def k_fold_cross_validation(k_folds: list,
             _model = deepcopy(model)
             _model.fit(dataset_train)
             preds_test = _model.predict(dataset_test)
-            preds.append(preds_test)
-            scores.append(r2_score(dataset_test.y, preds_test))
         elif isinstance(model, PyTorchModel):
             min_loss_val = 1e+8
             dataset_train, dataset_val = dataset_train.split(ratio_train=0.8, random_seed=random_seed)
@@ -195,9 +197,22 @@ def k_fold_cross_validation(k_folds: list,
                     min_loss_val = loss_val
 
             model.load_state_dict(best_state_dict)
-
             preds_test = model.predict(dataset_test)
-            preds.append(preds_test)
             scores.append(criterion(preds_test, dataset_test.y).item())
+        else:
+            raise KeyError('Unknown source library \'{}\' of the prediction model.'.format(model.alg_src))
 
-    return torch.vstack(preds), numpy.mean(scores), numpy.std(scores)
+        preds.append(preds_test)
+        targets.append(dataset_test.y)
+        tooltips.append(dataset_test.tooltips)
+
+    preds = torch.vstack(preds)
+    targets = torch.vstack(targets)
+    eval_results['preds_test'] = preds.flatten().tolist()
+    eval_results['targets_test'] = targets.flatten().tolist()
+    eval_results['tooltips_test'] = flatten_lists(tooltips)
+    eval_results['mae_test'] = mae(targets, preds)
+    eval_results['rmse_test'] = rmse(targets, preds)
+    eval_results['r2_test'] = r2_score(targets, preds)
+
+    return eval_results
